const Pager = require('./pager');

module.exports = function (domain, user, password) {
  var url = 'https://' + domain + '.myspringboard.us/api/';
  var request = require('./rest-request')(url, user, password);

  this.createInventoryAdjustment = function (locationId, itemId, adjustmentReasonId, qtyAdjust, cost, data, callback) {
    data = data || {};
    data['location_id'] = locationId;
    data['item_id'] = itemId;
    data['adjustment_reason_id'] = adjustmentReasonId;
    data['qty'] = qtyAdjust;
    data['unit_cost'] = cost;
    request('inventory/adjustments', 'postJson', data, callback);
  }

  this.createItem = function (cost, data, callback) {
    data = data || {};
    data['cost'] = cost;
    request('items', 'postJson', data, callback);
  };

  this.createItemVendor = function (itemId, vendorId, data, callback) {
    data = data || {};
    data['item_id'] = itemId;
    data['vendor_id'] = vendorId;
    request('items/' + itemId + '/vendors', 'postJson', data, callback);
  };


  // Order
  //   id: integer(required)
  //   customer_id: integer(required)
  //   station_id: integer(required)
  //   source_location_id: integer(required)
  //   status: text The status of the order.Valid options: "pending", "open", "canceled", "closed"
  //   sales_rep: text
  //   recalculate: boolean
  //   created_at: DateTime The date and time that this record was created
  //   updated_at: DateTime The date and time that this record was last updated
  //   shipping_method_id: integer
  //   shipping_charge: numeric(10, 2)
  //   billing_address_id: integer
  //   shipping_address_id: integer

  // OrderLine
  //    id: integer(required)
  //    item_id: integer(required)
  //    description: text
  //    qty: numeric(10, 2)(required)
  //    adjusted_unit_price: numeric(10, 2)
  //    original_unit_price: numeric(10, 2)
  //    unit_cost: numeric(10, 2)
  //    total_tax: numeric(10, 2) Extended tax amount to collect for the line
  //    created_at:DateTime The date and time that this record was created
  //    updated_at: DateTime The date and time that this record was last updated
  //    qty_open: numeric(10, 2)
  //    qty_invoiced: numeric(10, 2)
  //    order_id: integer
  //    ship_from_location_id: integer

  // OrderPayment
  //    id: integer(required)
  //    type: text(required) The payment type.Valid options: "CashPayment", "CreditCardPayment", "CheckPayment", "CustomPayment", "ExternalPayment"
  //    status: text The payment 's current status indicating whether or not it has been captured.
  //    deposit: boolean(required) If true, the funds are captured immediately.Otherwise funds are captured when fulfilling the order.
  //    amount:numeric(10, 2)(required)The amount of money to pay.
  //    created_at: DateTime The date and time that this record was created
  //    updated_at: DateTime The date and time that this record was last updated

  this.createOrder = function (customerId, locationId, stationId, data, callback) {
    var data = data || {};
    data['customer_id'] = customerId;
    data['source_location_id'] = locationId;
    data['station_id'] = stationId;
    request('sales/orders', 'postJson', data, callback);
  };

  this.getOrder = function (orderId, callback) {
    var data = {};
    request('sales/orders/' + orderId, 'get', data, callback);
  };

  this.updateOrder = function (orderId, data, callback) {
    var data = data || {};
    request('sales/orders/' + orderId, 'putJson', data, callback);
  };

  this.createOrderLine = function (orderId, itemId, quantity, data, callback) {
    var data = data || {};
    data['item_id'] = itemId;
    data['qty'] = quantity;
    request('sales/orders/' + orderId + '/lines', 'postJson', data, callback);
  };

  this.getOrderLines = function (orderId, callback) {
    var data = {};
    request('sales/orders/' + orderId + '/lines', 'get', data, callback);
  };

  this.shipOrderLines = function (orderId, orderLineId, shippingLocationId, callback) {
    data['ship_from_location_id'] = shippingLocationId;
    request('sales/orders/' + orderId + '/lines/' + orderLineId, 'putJson', data, callback);
  };

  this.createOrderShipping = function (orderId, shippingMethodId, shippingCharge, data, callback) {
    var data = data || {};
    data['shipping_method_id'] = shippingMethodId;
    data['shipping_charge'] = shippingCharge;
    request('sales/orders/' + orderId, 'putJson', data, callback);
  };

  this.getOrderPayment = function (orderId, callback) {
    var data = {};
    request('sales/orders/' + orderId + '/payments', 'get', data, callback);
  };

  this.createOrderPayment = function (orderId, data, callback) {
    var data = data || {};
    request('sales/orders/' + orderId + '/payments', 'postJson', data, callback);
  };

  this.getOrdersByFilter = function (jsonFilter, callback) {
    filterPager = new Pager(getOrdersByFilterRaw, convertArgsToArray(arguments));
  };

  this.getOrdersByFilterRaw = function (pageNumber, perPage, jsonFilter, pager) {
    request('sales/orders?_filter=' + encodeURIComponent(JSON.stringify(jsonFilter)) + "&per_page=" + perPage + "&page=" + pageNumber, 'get', {}, pager);
  };

  this.getPaymentTypes = function (callback) {
    var data = {};
    request('payment_types', 'get', data, callback);
  };

  // Invoice
  //    id: integer(required)
  //    metadata: text
  //    type: text(required)
  //    customer_id: integer
  //    source_location_id: integer
  //    sales_rep: text
  //    created_at: DateTime The date and time that this record was created
  //    updated_at: DateTime The date and time that this record was last updated
  //    public_id: text
  //    station_id: integer
  //    parent_transaction_id: integer
  //    recalculate: boolean
  //    status: text
  //    order_id: integer
  //    total: numeric(10, 2)
  //    created_by_user_id: integer

  this.createInvoice = function (data, callback) {
    var data = data || {};
    request('sales/invoices', 'postJson', data, callback);
  };

  this.getInvoice = function (invoiceId, callback) {
    var data = {};
    request('sales/invoices/' + invoiceId, 'get', data, callback);
  };

  this.getInvoiceLines = function (invoiceId, callback) {
    var data = {};
    request('sales/invoices/' + invoiceId + '/lines', 'get', data, callback);
  };

  this.createInvoiceLines = function (invoiceId, itemId, qty, type, callback) {
    var data = {
      'item_id': itemId,
      'qty': qty,
      'type': type
    };
    request('sales/invoices/' + invoiceId + '/lines', 'postJson', data, callback);
  };

  this.updateInvoice = function (invoiceId, data, callback) {
    var data = data || {};
    request('sales/invoices/' + invoiceId, 'putJson', data, callback);
  };

  this.getInvoicesByFilter = function (jsonFilter, callback) {
    filterPager = new Pager(getInvoicesByFilterRaw, convertArgsToArray(arguments));
  };

  this.getInvoicesByFilterRaw = function (pageNumber, perPage, jsonFilter, pager) {
    request('sales/invoices?_filter=' + encodeURIComponent(JSON.stringify(jsonFilter)) + "&per_page=" + perPage + "&page=" + pageNumber, 'get', {}, pager);
  };

  // Customer
  //   id: integer(required)
  //   public_id: text Unique ID string
  //   first_name: text First name
  //   last_name: text Last name
  //   email: text Email address
  //   address_id: integer ID of default address record
  //   shipping_address_id: integer ID of address record used for shipping
  //   billing_address_id:integer ID of address record used for billing
  //   active:BooleanWhether or not this record is currently active and being used
  //   customer_import_batch_id: integer ID of batch if thisrecordwascreated as part of an import batch
  //   custom: object Custom field values
  //   created_at: DateTime The date and time that this record was created
  //   updated_at: DateTime The date and time that this record was last updated

  this.createCustomer = function (data, callback) {
    var data = data || {};
    request('customers', 'postJson', data, callback);
  };

  this.getCustomer = function (customerId, callback) {
    var data = {};
    request('customers/' + customerId, 'get', data, callback);
  };

  this.updateCustomer = function (customerId, data, callback) {
    var data = data || {};
    request('customers/' + customerId, 'putJson', data, callback);
  };

  this.getCustomersByFilter = function (jsonFilter, callback) {
    filterPager = new Pager(getCustomersByFilterRaw, convertArgsToArray(arguments));
  };

  this.getCustomersByFilterRaw = function (pageNumber, perPage, jsonFilter, pager) {
    request('customers?_filter=' + encodeURIComponent(JSON.stringify(jsonFilter)) + "&per_page=" + perPage + "&page=" + pageNumber, 'get', {}, pager);
  };


  // Address
  //    id: integer
  //    address_id: integer
  //    current_revision_id: integer
  //    created_at: DateTime The date and time that this record was created
  //    updated_at: DateTime The date and time that this record was last updated
  //    type: text
  //    first_name: text
  //    last_name: text
  //    line_1: text
  //    line_2: text
  //    city: text
  //    state: text If the country is set to 'US' then 2 letter state acronym required
  //    phone: text
  //    country: text Set to 2 letter country acronym.Set value to anything other than 'US' to use a non standard state / postal_code
  //    postal_code: text If the country is set to 'US' then a 5 digit ZIP or ZIP + 4 code is required
  //    custom: object Custom field values

  this.createCustomerAddress = function (customerId, data, callback) {
    var data = data || {};
    request('customers/' + customerId + '/addresses', 'postJson', data, callback);
  };

  this.getCustomerAddresses = function (customerId, callback) {
    var data = {};
    request('customers/' + customerId + '/addresses', 'get', data, callback);
  };

  this.getCustomerAddress = function (customerId, addressId, callback) {
    var data = {};
    request('customers/' + customerId + '/addresses/' + addressId, 'get', data, callback);
  };

  this.updateCustomerAddress = function (customerId, addressId, data, callback) {
    var data = data || {};
    request('customers/' + customerId + '/addresses/' + addressId, 'putJson', data, callback);
  };


  // Ticket
  this.createTicket = function (data, callback) {
    data = data || {};
    request('sales/tickets', 'postJson', data, callback);
  };

  this.createTicketLine = function (ticketId, data, callback) {
    data = data || {};
    request('sales/tickets/' + ticketId + '/lines', 'postJson', data, callback);
  };

  this.createVendor = function (name, data, callback) {
    data = data || {};
    data['name'] = name;
    request('purchasing/vendors', 'postJson', data, callback);
  };

  this.createVendorAddress = function (vendorId, city, state, country, postalCode, data, callback) {
    data = data || {};
    data['type'] = 'location'; // Vendor addresses are all of type location
    data['city'] = city;
    data['state'] = state;
    data['country'] = country;
    data['postal_code'] = postalCode;
    request('purchasing/vendors/' + vendorId + '/address', 'postJson', data, callback);
  };

  this.createVendorContact = function (vendorId, data, callback) {
    request('purchasing/vendors/' + vendorId + '/contact', 'postJson', data, callback);
  };

  this.getCustomField = function (id, callback) {
    request('custom_fields/' + id, 'get', {}, callback);
  };

  this.getCustomFieldsByFilter = function (jsonFilter, callback) {
    filterPager = new Pager(getCustomFieldsByFilterRaw, convertArgsToArray(arguments));
  };

  this.getCustomFieldsByFilterRaw = function (pageNumber, perPage, jsonFilter, pager) {
    request('custom_fields?_filter=' + encodeURIComponent(JSON.stringify(jsonFilter)) + "&per_page=" + perPage + "&page=" + pageNumber, 'get', {}, pager);
  };

  this.getDailyItemMovement = function (jsonFilter, callback) {
    filterPager = new Pager(getDailyItemMovementRaw, convertArgsToArray(arguments));
  };

  this.getDailyItemMovementRaw = function (pageNumber, perPage, jsonFilter, pager) {
    request('reporting/analyzer?metrics[]=source_sales.net_sales&metrics[]=purchasing_net.delta_qty&metrics[]=inventory_adjustment.delta_qty&metrics[]=inventory_transfer_received.delta_qty&metrics[]=inventory_transfer_shipped.delta_qty&metrics[]=source_sales.total_cost&metrics[]=source_sales.gross_margin_ratio&metrics[]=source_sales.net_qty_sold&metrics[]=source_sales.gross_margin&metrics[]=source_sales.gross_returns&groups[]=date.date&groups[]=location.id&groups[]=item.id&groups[]=item.custom@dynamic_gp&start_date=' + encodeURIComponent(jsonFilter.start_date) + '&end_date=' + encodeURIComponent(jsonFilter.end_date) + '&per_page=' + perPage + '&page=' + pageNumber, 'get', {}, pager);
  };

  this.getDailyTicketLines = function (jsonFilter, callback) {
    filterPager = new Pager(getDailyTicketLinesRaw, convertArgsToArray(arguments));
  };

  this.getDailyTicketLinesRaw = function (pageNumber, perPage, jsonFilter, pager) {
    request('reporting/analyzer?metrics[]=source_sales.net_sales&metrics[]=source_sales.total_cost&metrics[]=source_sales.gross_margin_ratio&metrics[]=source_sales.net_qty_sold&metrics[]=source_sales.gross_margin&metrics[]=source_sales.gross_returns&groups[]=date.date&groups[]=location.id&groups[]=item.id&groups[]=sales_ticket.number&groups[]=sales_ticket.id&start_date=' + encodeURIComponent(jsonFilter.start_date) + '&end_date=' + encodeURIComponent(jsonFilter.end_date) + '&per_page=' + perPage + '&page=' + pageNumber, 'get', {}, pager);
  };

  this.getGiftCard = function (giftCardNumber, callback) {
    request('gift_cards/' + giftCardNumber, 'get', {}, callback);
  };

  this.getInventoryAdjustmentsByFilter = function (jsonFilter, callback) {
    filterPager = new Pager(getInventoryAdjustmentsByFilterRaw, convertArgsToArray(arguments));
  };

  this.getInventoryAdjustmentsByFilterRaw = function (pageNumber, perPage, jsonFilter, pager) {
    request('inventory/adjustments?_filter=' + encodeURIComponent(JSON.stringify(jsonFilter)) + "&per_page=" + perPage + "&page=" + pageNumber, 'get', {}, pager);
  };

  this.getInventoryValues = function (data, callback) {
    data = data || {};
    request('inventory/values', 'get', data, callback);
  };

  this.getInventoryValuesByFilter = function (jsonFilter, callback) {
    filterPager = new Pager(getInventoryValuesByFilterRaw, convertArgsToArray(arguments));
  };

  this.getInventoryValuesByFilterRaw = function (pageNumber, perPage, jsonFilter, pager) {
    request('inventory/values?_filter=' + encodeURIComponent(JSON.stringify(jsonFilter)) + "&per_page=" + perPage + "&page=" + pageNumber + '&group[]=location_id&group[]=item_id', 'get', {}, pager);
  };

  this.getInventoryValuesByLocation = function (itemId, data, callback) {
    data = data || {};
    data['item_id'] = itemId;
    data['group[]'] = 'location_id';
    data['_include[]'] = 'location';
    request('inventory/values', 'get', data, callback)
  };

  this.getItem = function (id, callback) {
    request('items/' + id + '?_include[]=primary_vendor&_include[]=vendor_details', 'get', {}, callback);
  };

  this.getItemByLookup = function (lookup, callback) {
    request('items/lookup_code=' + lookup, 'get', {}, callback);
  };

  this.getItemsByFilter = function (jsonFilter, callback) {
    filterPager = new Pager(getItemsByFilterRaw, convertArgsToArray(arguments));
  };

  this.getItemsByFilterRaw = function (pageNumber, perPage, jsonFilter, pager) {
    request('items?_filter=' + encodeURIComponent(JSON.stringify(jsonFilter)) + "&per_page=" + perPage + "&page=" + pageNumber + '&_include[]=primary_vendor&_include[]=vendor_details', 'get', {}, pager);
  };

  this.getItemVendors = function (id, callback) {
    request('items/' + id + '/vendors', 'get', {}, callback);
  };

  this.getPurchaseOrder = function (id, callback) {
    request('purchasing/orders/' + id, 'get', {}, callback);
  };

  this.getPurchaseOrderLines = function (id, callback) {
    request('purchasing/orders/' + id + '/lines?per_page=1000', 'get', {}, callback);
  };

  this.getPurchaseOrdersByFilter = function (jsonFilter, callback) {
    filterPager = new Pager(getPurchaseOrdersByFilterRaw, convertArgsToArray(arguments));
  };

  this.getPurchaseOrdersByFilterRaw = function (pageNumber, perPage, jsonFilter, pager) {
    request('purchasing/orders?_filter=' + encodeURIComponent(JSON.stringify(jsonFilter)) + "&per_page=" + perPage + "&page=" + pageNumber, 'get', {}, pager);
  };

  this.getTemplate = function (id, callback) {
    request('templates/' + id + '.html.liquid', 'get', {}, callback);
  };

  this.getTicket = function (id, callback) {
    request('sales/tickets/' + id + '?_include[]=customer', 'get', {}, callback);
  };

  this.getTicketLines = function (id, callback) {
    request('sales/tickets/' + id + '/lines?_include[]=item&per_page=1000', 'get', {}, callback);
  };

  this.getTicketsByFilter = function (jsonFilter, callback) {
    filterPager = new Pager(getTicketsByFilterRaw, convertArgsToArray(arguments));
  };

  this.getTicketsByFilterRaw = function (pageNumber, perPage, jsonFilter, pager) {
    request('sales/tickets?_filter=' + encodeURIComponent(JSON.stringify(jsonFilter)) + "&per_page=" + perPage + "&page=" + pageNumber + "&_include[]=customer", 'get', {}, pager);
  };

  this.getUsersByFilter = function (jsonFilter, callback) {
    filterPager = new Pager(getUsersByFilterRaw, convertArgsToArray(arguments));
  };

  this.getUsersByFilterRaw = function (pageNumber, perPage, jsonFilter, pager) {
    request('users?_filter=' + encodeURIComponent(JSON.stringify(jsonFilter)) + "&per_page=" + perPage + "&page=" + pageNumber, 'get', {}, pager);
  };

  this.getVendor = function (id, callback) {
    request('purchasing/vendors/' + id, 'get', {}, callback);
  };

  this.getVendorsByFilter = function (jsonFilter, callback) {
    filterPager = new Pager(getVendorsByFilterRaw, convertArgsToArray(arguments));
  };

  this.getVendorsByFilterRaw = function (pageNumber, perPage, jsonFilter, pager) {
    request('purchasing/vendors?_filter=' + encodeURIComponent(JSON.stringify(jsonFilter)) + "&per_page=" + perPage + "&page=" + pageNumber, 'get', {}, pager);
  };

  // Searches items based on a search term
  this.searchItems = function (searchTerm, page, perPage, callback) {
    data = { 'query': searchTerm.replace(' ', '+'), 'page': page, 'per_page': perPage };
    request('items', 'get', data, callback);
  };

  // Searches vendors based on a search term
  this.searchVendors = function (searchTerm, page, perPage, callback) {
    data = { 'query': searchTerm.replace(' ', '+'), 'page': page, 'per_page': perPage };
    request('vendors', 'get', data, callback);
  };

  this.updateCustomUrls = function (urls, callback) {
    request('settings/custom_javascript_urls', 'putJson', { 'value': urls }, callback);
  };

  this.updateItem = function (id, data, callback) {
    request('items/' + id, 'putJson', data, callback);
  };

  this.updateTemplate = function (id, data, callback) {
    request('templates/' + id + '.html.liquid', 'putJson', data, callback);
  };

  this.updateTicket = function (id, data, callback) {
    request('sales/tickets/' + id, 'putJson', data, callback);
  };

  this.updateVendor = function (id, data, callback) {
    request('purchasing/vendors/' + id, 'putJson', data, callback);
  };

  this.updateVendorAddress = function (vendorId, data, callback) {
    request('purchasing/vendors/' + vendorId + '/address', 'putJson', data, callback);
  };

  return this;
}

function convertArgsToArray(toConvert) {
  arr = [];
  for (i = 0; i < toConvert.length; i++) {
    arr.push(toConvert[i]);
  }

  return arr.sort();
}